module hecuba_exporter
!---------------------------------------------------------------------
!
!  Module containing definitions of variables needed to
!  store the 2D and 3D vars in Hecuba
!
!---------------------------------------------------------------------

   use forpy_mod
   !use, intrinsic :: iso_fortran_env, only: int64, int32, real32, real64
   implicit none
   !save

   contains

     subroutine def_stream2D_hecuba(glsize, lcsize, name, description, units, freq, freq_unit, accuracy)
       !use mod_mesh
       !use g_PARSUP
       use forpy_mod
       implicit none

       ! parameters
       integer,               intent(in)    :: glsize, lcsize
       character(len=*),      intent(in)    :: name, description, units
       ! real(kind=WP), target, intent(inout) :: data(:)
       integer,               intent(in)    :: freq
       character,             intent(in)    :: freq_unit
       integer,               intent(in)    :: accuracy
       ! type(Meandata),        allocatable :: tmparr(:)
       ! type(Meandata),        pointer     :: entry
       !type(t_mesh), intent(in), target    :: mesh

       !!!!! Forpy/Python related variables
       integer :: ierror
       type(tuple) :: args
       ! type(dict) :: kwargs
       type(module_py) :: mymodule
       type(object) :: return_value
       type(list) :: paths
       type(dict) :: data

       character(len=:), allocatable :: return_string

       ! init Forpy lib
       ierror = forpy_initialize()

       ! Instead of setting the environment variable PYTHONPATH,
       ! we can add the current directory "." to sys.path
       ierror = get_sys_path(paths)
       ierror = paths%append(".")

       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
       ! load Python module to be used against Hecuba DB
       ! it is assumed that the file hecuba_exporter.py is in the same path where is the compiled code
       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

       ierror = import_py(mymodule, "hecuba_exporter")
       ! it needs to be created even if it is empty
       ierror = tuple_create(args, 0)

       ! passing the vars
       ! # call def_stream(nod2D, myDim_nod2D, 'sst', 'sea surface temperature', 'C', tr_arr(1,1:myDim_nod2D,1), io_list(i)%freq, io_list(i)%unit, io_list(i)%precision, mesh)
       ierror = dict_create(data)
       ierror = data%setitem("glsize", glsize)
       ierror = data%setitem("lcsize", lcsize)
       ierror = data%setitem("name", name)
       ierror = data%setitem("description", description)
       ierror = data%setitem("units", units)
       ierror = data%setitem("freq", freq)
       ierror = data%setitem("freq_unit", freq_unit)
       ierror = data%setitem("accuracy", accuracy)

       ! call to store in Hecuba
       ierror = call_py(return_value, mymodule, "store_data", args, data)
       ierror = cast(return_string, return_value)

       write(*,*) return_string

       ! Clean up all modules & vars used
       call data%destroy
       call mymodule%destroy
       call return_value%destroy
       call paths%destroy
       call forpy_finalize

	   end subroutine def_stream2D_hecuba

	  !function def_stream3D_hecuba

	  !end function def_stream3D_hecuba


end module hecuba_exporter